<?php
//////////////////////////////////////////////////////////////
// Copyright (C) 2009 : Tangui Morlier, Benjamin Ooghe, Collectif
// This file is part of Regards Citoyens' CPC Project.
// The Project is free software: you can redistribute it 
// and/or modify it under the terms of the Affero General
// Public License as published by Free Software Foundation
// either version 3 of the License, or (at your option)
// any later version.

// The Project is distributed in the hope that it will be
// useful, but WITHOUT ANY WARRANTY; without even the 
// implied warranty of MERCHANTABILITY or FITNESS FOR 
// A PARTICULAR PURPOSE. See the Affero General Public
// License for more details.

// You should have received a copy of the Affero General
// Public
// License in the COPYING file that comes with Regards 
// Citoyen's CPC Project.
// If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------?>
<?php

class topDeputesTask extends sfBaseTask
{
  protected function configure()
  {
    $this->namespace = 'top';
    $this->name = 'Deputes';
    $this->briefDescription = 'Top Deputes';
  }
 
  protected function orderDeputes($type, $reverse = 1) {
    $tot = 0;
    foreach(array_keys($this->deputes) as $id) {
      if (!isset($this->deputes[$id][$type]['value'])) 
	$this->deputes[$id][$type]['value'] = 0;
      $ordered[$id] = $this->deputes[$id][$type]['value'];
      $tot++;
    }
    if ($reverse)
      arsort($ordered);
    else
      asort($ordered);
      
    $cpt = 0;
    $last_value = -999;
    $last_cpt = 0;
    foreach(array_keys($ordered) as $id) {
      $cpt++;
      if ($last_value != $this->deputes[$id][$type]['value'])
	$last_cpt = $cpt;
      $this->deputes[$id][$type]['rank'] = $last_cpt;
      $this->deputes[$id][$type]['max_rank'] = $tot;
      $last_value = $this->deputes[$id][$type]['value'];
    }
  }

  protected function executePresence($q)
  {
    $semaines = $q->select('p.id, s.numero_semaine, pr.id, count(s.id)')
      ->from('Parlementaire p, p.Presences pr, pr.Seance s')
      ->groupBy('p.id, s.numero_semaine')
      ->fetchArray();
    foreach ($semaines as $p) {
      foreach($p['Presences'] as $pr) {
	$this->deputes[$p['id']]['semaine']['value']++;
      }
    }
  }
  protected function executeCommissionPresence($q) 
  {
      $parlementaires = $q->select('p.id, count(pr.id)')
	->from('Parlementaire p, p.Presences pr, pr.Seance s')
	->groupBy('p.id')
	->andWhere('s.type = ?', 'commission')
	->fetchArray();
      foreach ($parlementaires as $p) {
	$this->deputes[$p['id']]['commission_presences']['value'] = $p['count'];
      }
  }
  protected function executeCommissionInterventions($q)
  {
      $parlementaires = $q->select('p.id, count(i.id)')
	->from('Parlementaire p, p.Interventions i')
	->groupBy('p.id')
	->andWhere('i.type = ?', 'commission')
	->fetchArray();
      foreach ($parlementaires as $p) {
	$this->deputes[$p['id']]['commission_interventions']['value'] = $p['count'];
      }
  }
  protected function executeHemicycleInvectives($q) 
  {
      $parlementaires = $q->select('p.id, count(i.id)')
	->from('Parlementaire p, p.Interventions i, i.Seance s')
	->groupBy('p.id')
	->andWhere('s.type = ?', 'hemicycle')
	->andWhere('i.nb_mots <= 20')
	->fetchArray();
      foreach ($parlementaires as $p) {
	$this->deputes[$p['id']]['hemicycle_invectives']['value'] = $p['count'];
      }
  }
  protected function executeHemicycleInterventions($q)
  {
    $parlementaires = $q->select('p.id, count(i.id)')
      ->from('Parlementaire p, p.Interventions i, i.Seance s')
      ->groupBy('p.id')
      ->andWhere('s.type = ?', 'hemicycle')
      ->andWhere('i.nb_mots > 20')
      ->fetchArray();
    foreach ($parlementaires as $p) {
      $this->deputes[$p['id']]['hemicycle_interventions']['value'] = $p['count'];
    }
  }

  protected function executeAmendementsSignes($q)
  {
    $parlementaires = $q->select('p.id, count(a.id)')
      ->from('Parlementaire p, p.Amendements a')
      ->groupBy('p.id')
      ->fetchArray();
    foreach ($parlementaires as $p) {
      $this->deputes[$p['id']]['amendements_signes']['value'] = $p['count'];
    }
  }
 
  protected function executeAmendementsAdoptes($q)
  {
    $parlementaires = $q->select('p.id, count(a.id)')
      ->from('Parlementaire p, p.Amendements a')
      ->groupBy('p.id')
      ->andWhere('a.sort = ?', 'Adopté')
      ->fetchArray();
    foreach ($parlementaires as $p) {
      $this->deputes[$p['id']]['amendements_adoptes']['value'] = $p['count'];
    }
  }

  protected function executeAmendementsRejetes($q)
  {
    $parlementaires = $q->select('p.id, count(a.id)')
      ->from('Parlementaire p, p.Amendements a')
      ->groupBy('p.id')
      ->andWhere('a.sort = ?', 'Rejeté')
      ->fetchArray();
    foreach ($parlementaires as $p) {
      $this->deputes[$p['id']]['amendements_rejetes']['value'] = $p['count'];
    }
  }
  
  protected function executeQuestionsEcrites($q)
  {
    $parlementaires = $q->select('p.id, count(q.id)')
      ->from('Parlementaire p, p.QuestionEcrites q')
      ->groupBy('p.id')
      ->fetchArray();
    foreach ($parlementaires as $p) {
      $this->deputes[$p['id']]['questions_ecrites']['value'] = $p['count'];
    }
  }
  protected function executeQuestionsOrales($q)
  {
    $questions = $q->select('p.id, count(DISTINCT i.seance_id) as count')
      ->from('Parlementaire p, p.Interventions i')
      ->groupBy('p.id')
      ->andWhere('i.type = ?', 'question')
      ->andWhere('i.nb_mots > 20')
      ->andWhere('i.fonction NOT LIKE ?', 'président%')
      ->fetchArray();
    foreach ($questions as $q) {
      $this->deputes[$q['id']]['questions_orales']['value'] = $q['count'];
    }
  }
  protected function execute($arguments = array(), $options = array())
  {
    $manager = new sfDatabaseManager($this->configuration);

    $this->deputes = array();
    
    $deputes = Doctrine::getTable('Parlementaire')->createQuery()
      ->where('type = ?', 'depute')
      ->andWhere('fin_mandat IS NULL') 
      ->fetchArray();
    foreach($deputes as $d) {
      $this->deputes[$d['id']]['groupe'] = $d['groupe_acronyme'];
    }

    $q = Doctrine_Query::create()->where('fin_mandat IS NULL');
 
    $qs = clone $q;
    $qs->andWhere('s.date > ?', date('Y-m-d', time()-60*60*24*365));

    
     
    $this->executePresence(clone $qs);
    $this->orderDeputes('semaine');
    
    $this->executeCommissionPresence(clone $qs);
    $this->orderDeputes('commission_presences');
    
    $qi = clone $q;
    $qi->andWhere('i.date > ?', date('Y-m-d', time()-60*60*24*365));

    $this->executeCommissionInterventions(clone $qi);
    $this->orderDeputes('commission_interventions');
    
    $this->executeHemicycleInterventions(clone $qi);
    $this->orderDeputes('hemicycle_interventions');
    

    $this->executeHemicycleInvectives(clone $qi);
    $this->orderDeputes('hemicycle_invectives');
    
    $qa = clone $q;
    $qa->andWhere('a.date > ?', date('Y-m-d', time()-60*60*24*365));
    $this->executeAmendementsSignes(clone $qa);
    $this->orderDeputes('amendements_signes');
    
    $this->executeAmendementsAdoptes(clone $qa);
    $this->orderDeputes('amendements_adoptes');

    $this->executeAmendementsRejetes(clone $qa);
    $this->orderDeputes('amendements_rejetes', 0);

    $qq = clone $q;
    $qq->where('q.date > ?', date('Y-m-d', time()-60*60*24*365));
    $this->executeQuestionsEcrites($qq);
    $this->orderDeputes('questions_ecrites');

    $this->executeQuestionsOrales(clone $qi);
    $this->orderDeputes('questions_orales');


    $groupes = array();
    foreach(array_keys($this->deputes) as $id) {
      foreach(array_keys($this->deputes[$id]) as $key) {
	$groupes[$this->deputes[$id]['groupe']][$key]['somme'] += $this->deputes[$id][$key]['value'];
	$groupes[$this->deputes[$id]['groupe']][$key]['nb']++;
      }
      unset($this->deputes[$id]['groupe']);
      $depute = Doctrine::getTable('Parlementaire')->find($id);
      $depute->top = serialize($this->deputes[$id]);
      $depute->save();
    }

    $globale = doctrine::getTable('VariableGlobale')->findOneByChamp('stats_groupes');
    if (!$globale) {
      $globale = new VariableGlobale();
      $globale->champ = 'stats_groupes';
    }
    $globale->value = serialize($groupes);
    $globale->save();

    //On fait la même chose pour les députés ayant un mandat clos.

    $parlementaires = Doctrine_Query::create()->where('fin_mandat IS NOT NULL')
      ->from('Parlementaire p')->execute();
    
    foreach ($parlementaires as $p) {
      $this->depute = array();
      $q = Doctrine_Query::create()->where('p.id = ?', $p->id);
      
      $qs = clone $q;
      $qs->andWhere('(s.date > ? AND s.date < ?)', array(date('Y-m-d', strtotime($p->debut_mandat)), 
       date('Y-m-d', strtotime($p->fin_mandat)), ));
     
      $this->executePresence(clone $qs);
      
      $this->executeCommissionPresence(clone $qs);
    
      $qi = clone $q;
      $qi->andWhere('(i.date > ? AND i.date < ?)', array(date('Y-m-d', strtotime($p->debut_mandat)), 
							 date('Y-m-d', strtotime($p->fin_mandat)), ));
     
      $this->executeCommissionInterventions(clone $qi);
      
      $this->executeHemicycleInterventions(clone $qi);
    

      $this->executeHemicycleInvectives(clone $qi);
    
      $qa = clone $q;
      $qa->andWhere('(a.date > ? AND a.date < ?)', array(date('Y-m-d', strtotime($p->debut_mandat)), 
							 date('Y-m-d', strtotime($p->fin_mandat)), ));
     

      $this->executeAmendementsSignes(clone $qa);
      
      $this->executeAmendementsAdoptes(clone $qa);
      
      $this->executeAmendementsRejetes(clone $qa);
      
      $qq = clone $q;
      $qq->andWhere('(q.date > ? AND q.date < ?)', array(date('Y-m-d', strtotime($p->debut_mandat)), 
							 date('Y-m-d', strtotime($p->fin_mandat)), ));
     
      $this->executeQuestionsEcrites($qq);
      
      $this->executeQuestionsOrales(clone $qi);

      if (count($this->deputes[$p->id])) {
	$p->top = serialize($this->deputes[$p->id]);
	$p->save();
      }
    }
    
  }
}
